package service

import (
	"fmt"
	"iot_manager_service/app/device/dao"
	"iot_manager_service/app/device/model"
	"iot_manager_service/app/system/service"
	"iot_manager_service/util"
	"time"
)

var GarbageWayGroupService = new(garbageWayGroupService)

type garbageWayGroupService struct{}

func (s *garbageWayGroupService) Get(id int) (*dao.GarbageWayGroup, *util.Errors) {
	// 创建查询实例
	device := &dao.GarbageWayGroup{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	return device, nil

}

func (s *garbageWayGroupService) CreateOrUpdate(userId int64, tenantId int, req dao.GarbageWayGroup) *util.Errors {
	// 创建查询实例
	device := req
	device.TenantId = tenantId
	device.UpdateUser = userId
	device.UpdateTime = time.Now()
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = userId
		if device.IsExistedByName() {
			fmt.Printf("Create GetDeviceID err \n")
			return util.ParamsInvalidResponse("存在重名,请更改名称！", nil)
		}
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return util.FailResponse(err.Error(), nil)
		}
		service.OperationHisService.Save(userId, tenantId, util.OperationCreate, util.ModuleTypeDevice,
			util.DeviceTypeGarbageGroup, util.GetDeviceObject(device.ID, device.GroupName), util.OperationSuccess)
		return util.SuccessResponse(util.Succeeded, nil)
	}
	if device.IsExistedByNameAndCode() {
		return util.ParamsInvalidResponse("列表中存在重名,请更改垃圾桶道路名称！", nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return util.FailResponse(err.Error(), nil)
	}

	service.OperationHisService.Save(userId, tenantId, util.OperationUpdate, util.ModuleTypeDevice,
		util.DeviceTypeGarbageGroup, util.GetDeviceObject(device.ID, device.GroupName), util.OperationSuccess)
	return util.SuccessResponse(util.Succeeded, nil)
}

func (s *garbageWayGroupService) List(searchValue string, current, size int) ([]dao.GarbageWayGroup, *util.Errors) {
	device := dao.GarbageWayGroup{}
	if searchValue != "" {
		device.GroupName = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	return devices, nil
}

func (s *garbageWayGroupService) Remove(userId int64, tenantId int, id int) *util.Errors {
	// 创建查询实例
	device := &dao.GarbageWayGroup{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: userId,
		UpdateTime: time.Now(),
	}
	err := device.Delete()
	if err != nil {
		return util.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, util.OperationRemove, util.ModuleTypeDevice,
		util.DeviceTypeGarbageGroup, util.GetDeviceObject(device.ID, device.GroupName), util.OperationSuccess)
	return nil
}

func (s *garbageWayGroupService) GetList(tenantId int) ([]*dao.GarbageWayGroup, *util.Errors) {
	var devices []*dao.GarbageWayGroup
	err := util.Redis.Get(getGarbageWayGroupListRedisKey("000000")).Scan(&devices)
	if err == nil {
		return devices, nil
	}
	device := &dao.GarbageWayGroup{
		TenantId:  tenantId,
		IsDeleted: 0,
	}
	devices, err = device.GetAllDevices()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	_ = util.Redis.Set(getGarbageWayGroupListRedisKey("000000"), devices, 0).Err()
	return devices, nil
}

func getGarbageWayGroupListRedisKey(tenantId string) string {
	return fmt.Sprintf(model.GarbageWayGroupList, tenantId)
}
