package service

import (
	"fmt"
	"iot_manager_service/app/device/dao"
	"iot_manager_service/app/device/model"
	"iot_manager_service/app/system/service"
	"iot_manager_service/util"
	"time"
)

var InfoBoardService = new(infoBoardService)

type infoBoardService struct{}

func (s *infoBoardService) Get(id int) (*model.InfoBoardDetail, *util.Errors) {
	// 创建查询实例
	device := &dao.InfoBoard{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	endTime, state := util.GetDeviceState(device.Sn)
	return &model.InfoBoardDetail{InfoBoard: *device,
		RunState:     state,
		NetworkState: state,
		EndLineTime:  endTime,
	}, nil
}

func (s *infoBoardService) CreateOrUpdate(userId int64, tenantId int, req dao.InfoBoard) *util.Errors {
	// 创建查询实例
	device := req
	device.TenantId = tenantId
	device.UpdateUser = userId
	device.UpdateTime = time.Now()
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = userId
		if device.IsExistedBySN() {
			fmt.Printf("Create GetDeviceID err \n")
			return util.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return util.FailResponse(err.Error(), nil)
		}
		service.OperationHisService.Save(userId, tenantId, util.OperationCreate, util.ModuleTypeDevice,
			util.DeviceTypeInfoBoard, util.GetDeviceObject(device.ID, device.InfoName), util.OperationSuccess)
		return util.SuccessResponse(util.Succeeded, nil)
	}
	if device.IsExistedByNameAndCode() {
		return util.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return util.FailResponse(err.Error(), nil)
	}

	service.OperationHisService.Save(userId, tenantId, util.OperationUpdate, util.ModuleTypeDevice,
		util.DeviceTypeInfoBoard, util.GetDeviceObject(device.ID, device.InfoName), util.OperationSuccess)
	return util.SuccessResponse(util.Succeeded, nil)
}

func (s *infoBoardService) List(searchValue string, current, size int) ([]model.InfoBoardDetail, *util.Errors) {
	device := dao.InfoBoard{}
	if searchValue != "" {
		device.Sn = searchValue
		device.InfoName = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	var details []model.InfoBoardDetail
	for _, d := range devices {
		endTime, state := util.GetDeviceState(d.Sn)
		details = append(details, model.InfoBoardDetail{
			InfoBoard:    d,
			RunState:     state,
			NetworkState: state,
			EndLineTime:  endTime,
		})
	}
	return details, nil
}

func (s *infoBoardService) Remove(userId int64, tenantId int, id int) *util.Errors {
	// 创建查询实例
	device := &dao.InfoBoard{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: userId,
		UpdateTime: time.Now(),
	}
	err := device.Delete()
	if err != nil {
		return util.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, util.OperationRemove, util.ModuleTypeDevice,
		util.DeviceTypeInfoBoard, util.GetDeviceObject(device.ID, device.InfoName), util.OperationSuccess)
	return nil
}

func (s *infoBoardService) GetByGateway(id int) []dao.InfoBoard {
	// 创建查询实例
	device := &dao.InfoBoard{
		GatewayId: id,
	}
	return device.GetDevicesByGateway()
}

func (s *infoBoardService) GetByLampPole(id int) []dao.InfoBoard {
	// 创建查询实例
	device := &dao.InfoBoard{
		LampPoleId: id,
	}
	return device.GetDevicesByLampPole()
}
